﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.


Namespace geometer

    Public NotInheritable Class Angle

        Private _rad As Double

        Public Enum angleunit
            radians = 0
            degrees = 1
            grads = 2
            revolutions = 3
        End Enum

        Public Sub New()
            _rad = 0.0R
        End Sub

        Public Sub New(radians As Double)
            _rad = radians
        End Sub

        Public Sub New(value As Double, unit As String)
            unit = Strings.Left(unit, 3).ToLower
            If unit = "rad" Then
                _rad = value
            ElseIf unit = "deg" Then
                _rad = value * Math.PI / 180.0R
            ElseIf unit = "gra" Then
                _rad = value * Math.PI / 200.0R
            ElseIf unit = "rev" Then
                _rad = value * Math.PI / 0.5R
            Else
                Throw New ApplicationException("Invalid Unit Argument in Constructor for Class Angle.  Unit must be 'rad', 'deg', 'gra', or 'rev'")
            End If

        End Sub

        Public Sub New(value As Double, unit As angleunit)
            If unit = angleunit.radians Then
                _rad = value
            ElseIf unit = angleunit.degrees Then
                _rad = value * Math.PI / 180.0R
            ElseIf unit = angleunit.grads Then
                _rad = value * Math.PI / 200.0R
            ElseIf unit = angleunit.revolutions Then
                _rad = value * Math.PI / 0.5R
            End If
        End Sub

        Public Property Radians As Double
            Get
                Return _rad
            End Get
            Set(value As Double)
                _rad = value
            End Set
        End Property

        Public Property Degrees As Double
            Get
                Return _rad * 180.0R / Math.PI
            End Get
            Set(value As Double)
                _rad = value * Math.PI / 180.0R
            End Set
        End Property

        Public Property Grads As Double
            Get
                Return _rad * 200.0R / Math.PI
            End Get
            Set(value As Double)
                _rad = value * Math.PI / 200.0R
            End Set
        End Property

        Public Property Revolutions As Double
            Get
                Return _rad * 0.5R / Math.PI
            End Get
            Set(value As Double)
                _rad = value * Math.PI / 0.5R
            End Set
        End Property

        Public ReadOnly Property Complement As Angle
            Get
                Return New Angle((Math.PI / 2) - _rad)
            End Get
        End Property

        Public ReadOnly Property Principal As Angle
            Get
                Dim myangle As Double = _rad Mod (2 * Math.PI)
                If myangle < 0 Then
                    myangle += (2 * Math.PI)
                End If
                Return New Angle(myangle)
            End Get
        End Property

        Public ReadOnly Property Supplement As Angle
            Get
                Return New Angle(Math.PI - _rad)
            End Get
        End Property

        Public Shared Operator +(angle1 As Angle, angle2 As Angle) As Angle
            Return New Angle(angle1.Radians + angle2.Radians)
        End Operator

        Public Shared Operator -(angle1 As Angle, angle2 As Angle) As Angle
            Return New Angle(angle1.Radians - angle2.Radians)
        End Operator

        Public Shared Operator -(angle0 As Angle) As Angle
            Return New Angle(-angle0.Radians)
        End Operator

        Public Shared Operator =(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians = angle2.Radians)
        End Operator

        Public Shared Operator <>(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians <> angle2.Radians)
        End Operator

        Public Shared Operator >(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians > angle2.Radians)
        End Operator

        Public Shared Operator <(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians < angle2.Radians)
        End Operator

        Public Shared Operator >=(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians >= angle2.Radians)
        End Operator

        Public Shared Operator <=(angle1 As Angle, angle2 As Angle) As Boolean
            Return (angle1.Radians <= angle2.Radians)
        End Operator

        Public Shared Operator *(factor As Double, angle1 As Angle) As Angle
            Return New Angle(factor * angle1.Radians)
        End Operator

        Public Shared Operator *(angle1 As Angle, factor As Double) As Angle
            Return New Angle(factor * angle1.Radians)
        End Operator

        Public Shared Operator /(angle1 As Angle, divisor As Double) As Angle
            Return New Angle(angle1.Radians / divisor)
        End Operator



    End Class

End Namespace



