﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class CMatrix 'entire class new in version 0.7.0

        Private _matrix(0, 0) As Complex

        Public Sub New()

        End Sub

        Public Sub New(rows As Integer, cols As Integer)
            ReDim _matrix(rows - 1, cols - 1)
        End Sub

        Public Sub New(size As Integer)
            ReDim _matrix(size - 1, size - 1)
        End Sub

        Public Sub New(matrixValues(,) As Complex)
            ReDim _matrix(matrixValues.GetLength(0) - 1, matrixValues.GetLength(1) - 1)
            _matrix = matrixValues
        End Sub

        Public Sub resize(rows As Integer, columns As Integer, preserve As Boolean)
            If preserve Then
                ReDim Preserve _matrix(rows - 1, columns - 1)
            Else
                ReDim _matrix(rows - 1, columns - 1)
            End If
        End Sub

        Public Property element(rowIndex As Integer, columnIndex As Integer) As Complex
            Get
                If Me.rows < rowIndex Or Me.columns < columnIndex Or rowIndex < 1 Or columnIndex < 1 Then
                    Throw New Exception("The indices you indicated are outside the bounds of the matrix")
                    Exit Property
                End If
                Return _matrix(rowIndex - 1, columnIndex - 1)
            End Get
            Set(value As Complex)
                If Me.rows < rowIndex Or Me.columns < columnIndex Or rowIndex < 1 Or columnIndex < 1 Then
                    Throw New Exception("The indices you indicated are outside the bounds of the matrix")
                    Exit Property
                End If
                _matrix(rowIndex - 1, columnIndex - 1) = value
            End Set
        End Property

        Public ReadOnly Property row(rowIndex As Integer) As CMatrix
            Get
                Dim inter As New CMatrix(1, Me.columns)
                For i As Integer = 1 To Me.columns
                    inter.element(1, i) = Me.element(rowIndex, i)
                Next
                Return inter
            End Get
        End Property

        Public ReadOnly Property column(columnIndex As Integer) As CMatrix
            Get
                Dim inter As New CMatrix(Me.rows, 1)
                For i As Integer = 1 To Me.rows
                    inter.element(i, 1) = Me.element(i, columnIndex)
                Next
                Return inter
            End Get
        End Property

        Public ReadOnly Property rowSum(rowIndex As Integer) As Complex
            Get
                Dim inter As New Complex(0.0R, 0.0R)
                For i As Integer = 1 To Me.columns
                    inter += Me.element(rowIndex, i)
                Next
                Return inter
            End Get
        End Property

        Public ReadOnly Property columnSum(columnIndex As Integer) As Complex
            Get
                Dim inter As New Complex(0.0R, 0.0R)
                For i As Integer = 1 To Me.rows
                    inter += Me.element(i, columnIndex)
                Next
                Return inter
            End Get
        End Property

        Public ReadOnly Property rows As Integer
            Get
                Return _matrix.GetLength(0)
            End Get
        End Property

        Public ReadOnly Property columns As Integer
            Get
                Return _matrix.GetLength(1)
            End Get
        End Property

        Public Sub swapRows(rowIndex1 As Integer, rowIndex2 As Integer)

            Dim hold As New CMatrix(1, Me.columns)
            For i As Integer = 1 To Me.columns
                hold.element(1, i) = Me.element(rowIndex1, i)
                Me.element(rowIndex1, i) = Me.element(rowIndex2, i)
                Me.element(rowIndex2, i) = hold.element(1, i)
            Next

        End Sub

        Public Sub swapColumns(columnIndex1 As Integer, columnIndex2 As Integer)

            Dim hold As New CMatrix(Me.rows, 1)
            For i As Integer = 1 To Me.rows
                hold.element(i, 1) = Me.element(i, columnIndex1)
                Me.element(i, columnIndex1) = Me.element(i, columnIndex2)
                Me.element(i, columnIndex2) = hold.element(i, 1)
            Next

        End Sub

        Public Sub addRows(rowIndexTarget As Integer, rowIndexSource As Integer, factor As Complex)

            If rowIndexSource = rowIndexTarget Then
                Throw New Exception("For the elementary operation of row addition, you cannot add a row to itself.  Use method CMatrix.multiplyRow instead.")
            End If

            If rowIndexTarget < 1 Or rowIndexSource < 1 Or rowIndexTarget > Me.rows Or rowIndexSource > Me.rows Then
                Throw New Exception("You have specified a row index that is out of bounds for the matrix.")
                Exit Sub
            End If

            For i As Integer = 0 To Me.columns - 1
                _matrix(rowIndexTarget - 1, i) = _matrix(rowIndexTarget - 1, i) + (factor * _matrix(rowIndexSource - 1, i))
            Next
        End Sub

        Public Sub addColumns(columnIndexTarget As Integer, columnIndexSource As Integer, factor As Complex)

            If columnIndexSource = columnIndexTarget Then
                Throw New Exception("For the elementary operation of column addition, you cannot add a column to itself.  Use method CMatrix.multiplyColumn instead.")
            End If

            If columnIndexTarget < 1 Or columnIndexSource < 1 Or columnIndexTarget > Me.columns Or columnIndexSource > Me.columns Then
                Throw New Exception("You have specified a column index that is out of bounds for the matrix.")
                Exit Sub
            End If

            For i As Integer = 0 To Me.rows - 1
                _matrix(i, columnIndexTarget - 1) = _matrix(i, columnIndexTarget - 1) + (factor * _matrix(i, columnIndexSource - 1))
            Next
        End Sub

        Public Sub multiplyRow(rowIndex As Integer, factor As Complex)
            If factor = New Complex(0.0R, 0.0R) Then
                Throw New Exception("Multiplication of a row by zero is invalid.")
                Exit Sub
            End If

            If rowIndex < 1 Or rowIndex > Me.rows Then
                Throw New Exception("You have specified a row index that is out of bounds for the matrix.")
                Exit Sub
            End If

            For i As Integer = 0 To Me.columns - 1
                _matrix(rowIndex - 1, i) = factor * _matrix(rowIndex - 1, i)
            Next

        End Sub

        Public Sub multiplyColumn(columnIndex As Integer, factor As Complex)
            If factor = New Complex(0.0R, 0.0R) Then
                Throw New Exception("Multiplication of a row by zero is invalid.")
                Exit Sub
            End If

            If columnIndex < 1 Or columnIndex > Me.columns Then
                Throw New Exception("You have specified a column index that is out of bounds for the matrix.")
                Exit Sub
            End If

            For i As Integer = 0 To Me.rows - 1
                _matrix(i, columnIndex - 1) = factor * _matrix(i, columnIndex - 1)
            Next

        End Sub

        Public Function determinant() As Complex

            If Me.rows <> Me.columns Then
                Throw New Exception("Determinants can be calculcated only for square matrices.")
                Exit Function
            End If

            Dim inter As Complex

            If Me.rows = 1 Then
                inter = Me.element(1, 1)
            ElseIf Me.rows = 2 Then
                inter = (Me.element(1, 1) * Me.element(2, 2)) - (Me.element(1, 2) * Me.element(2, 1))
            ElseIf Me.rows = 3 Then
                inter = (Me.element(1, 1) * Me.element(2, 2) * Me.element(3, 3)) + _
                       (Me.element(1, 2) * Me.element(2, 3) * Me.element(3, 1)) + _
                       (Me.element(1, 3) * Me.element(2, 1) * Me.element(3, 2)) - _
                       (Me.element(3, 1) * Me.element(2, 2) * Me.element(1, 3)) - _
                       (Me.element(3, 2) * Me.element(2, 3) * Me.element(1, 1)) - _
                       (Me.element(3, 3) * Me.element(2, 1) * Me.element(1, 2))
            Else
                Dim detsign As Integer = 1
                Dim detfactor As New Complex(1.0R, 0.0R)
                Dim elementavg As New Complex(0.0R, 0.0R)
                Dim work As New CMatrix(Me.rows, Me.columns)
                For irow As Integer = 1 To Me.rows
                    For icolumn As Integer = 1 To Me.columns
                        work.element(irow, icolumn) = Me.element(irow, icolumn)
                        elementavg += Me.element(irow, icolumn)
                    Next
                Next

                elementavg = elementavg / (CDbl(work.rows) ^ 2.0R)

                Dim eareal As Double = elementavg.Re

                Dim eaimg As Double = elementavg.Im
                If eareal > 0 Then
                    eareal = Math.Ceiling(eareal)
                Else
                    eareal = Math.Floor(eareal)
                End If

                If eaimg > 0 Then
                    eaimg = Math.Ceiling(eaimg)
                Else
                    eaimg = Math.Floor(eaimg)
                End If

                elementavg = New Complex(eareal, eaimg)

                If GMath.Abs(elementavg) < 1.0R Then
                    elementavg = New Complex(1.0R, 0.0R)
                End If

                'check to see that all values on diagonal are non-zero.  If not, correct it

                For idiag As Integer = 1 To work.rows
                    If work.element(idiag, idiag) = New Complex(0.0R, 0.0R) Then
                        If Not _resolveZero(work, idiag, elementavg) Then
                            Return 0.0R
                            Exit Function
                        End If
                    End If
                Next

                'now perform Gaussian elimination

                For icolumns As Integer = 1 To work.columns
                    _resolveOrder(work, icolumns, icolumns, detsign)
                    For irows As Integer = icolumns To work.rows
                        If irows = icolumns Then
                            detfactor *= work.element(irows, icolumns)
                            work.multiplyRow(irows, (1.0R / work.element(irows, icolumns)))
                        ElseIf work.element(icolumns, icolumns) <> 0 Then
                            work.addRows(irows, icolumns, -work.element(irows, icolumns))
                        End If
                    Next
                Next

                inter = CDbl(detsign) * detfactor

            End If

            Return inter

        End Function

        Public Function adjugate() As CMatrix

            If Me.rows <> Me.columns Then
                Throw New Exception("Adjugates can be calculated only for square matrices")
                Exit Function
            End If

            Dim tmatrix As CMatrix = Me.transpose
            Dim cadjugate As New CMatrix(Me.rows)
            Dim sign As Double

            For irow As Integer = 1 To tmatrix.rows
                sign = (-1.0R) ^ (CDbl((irow Mod 2) + 1))
                For icolumn As Integer = 1 To tmatrix.columns
                    cadjugate.element(irow, icolumn) = sign * _makeAdjugateElement(tmatrix, irow, icolumn)
                    sign *= -1.0R
                Next
            Next

            Return cadjugate

        End Function

        Public Function inverse() As CMatrix

            If Me.rows <> Me.columns Then
                Throw New Exception("Inverses can be calculated only for square matrices")
                Exit Function
            End If

            Return (1.0R / Me.determinant) * Me.adjugate

        End Function

        Public Function transpose() As CMatrix

            Dim tmatrix As New CMatrix(Me.columns, Me.rows)
            For row As Integer = 1 To Me.rows
                For column As Integer = 1 To Me.columns
                    tmatrix.element(column, row) = Me.element(row, column)
                Next
            Next

            Return tmatrix

        End Function

        Public Shared Function solveEquations(coefficientMatrix As Matrix, resultMatrix As Matrix) As Matrix

            Return coefficientMatrix.inverse * resultMatrix

        End Function

        Public Shared Function identity(size As Integer) As CMatrix
            Dim m As New CMatrix(size)
            For irow As Integer = 1 To size
                For icolumn As Integer = 1 To size
                    If irow = icolumn Then
                        m.element(irow, icolumn) = New Complex(1.0R, 0.0R)
                    Else
                        m.element(irow, icolumn) = New Complex(0.0R, 0.0R)
                    End If
                Next
            Next
            Return m
        End Function

        Private Function _resolveZero(ByRef work As CMatrix, ByRef idiag As Integer, elementavg As Complex) As Boolean

            If work.rowSum(idiag) = 0.0R Or work.columnSum(idiag) = 0.0R Then
                Return False
                Exit Function
            End If

            Dim maxrowindex As Integer = 1
            Dim maxrowvalue As Double = GMath.Abs(work.element(1, idiag))

            For i As Integer = 1 To work.rows
                If GMath.Abs(work.element(i, idiag)) > maxrowvalue Then
                    maxrowvalue = GMath.Abs(work.element(i, idiag))
                    maxrowindex = i
                End If
            Next

            work.addRows(idiag, maxrowindex, elementavg)

            Return True

        End Function

        Private Sub _resolveOrder(ByRef work As CMatrix, ByVal rowIndex As Integer, ByVal columnIndex As Integer, ByRef detsign As Integer)

            If rowIndex = work.rows Then Exit Sub

            Dim maxrowindex As Integer = rowIndex
            Dim maxrowvalue As Double = GMath.Abs(work.element(rowIndex, columnIndex))

            For i As Integer = (rowIndex + 1) To work.rows
                If GMath.Abs(work.element(i, columnIndex)) > maxrowvalue Then
                    maxrowvalue = GMath.Abs(work.element(i, columnIndex))
                    maxrowindex = i
                End If
            Next

            If maxrowindex > rowIndex Then
                work.swapRows(rowIndex, maxrowindex)
                detsign *= -1
            End If

        End Sub

        Private Function _makeAdjugateElement(ByRef tmatrix As CMatrix, ByRef irow As Integer, ByRef icolumn As Integer) As Complex

            Dim minor As New CMatrix(tmatrix.rows - 1)
            Dim minorelements(((tmatrix.rows) ^ 2) - 1) As Complex

            Dim arrayi As Integer = 0
            Dim jrow As Integer
            Dim jcolumn As Integer

            For jrow = 1 To tmatrix.rows
                For jcolumn = 1 To tmatrix.columns
                    If ((jrow <> irow) And (jcolumn <> icolumn)) Then
                        minorelements(arrayi) = tmatrix.element(jrow, jcolumn)
                        arrayi += 1
                    End If
                Next
            Next

            arrayi = 0

            For jrow = 1 To minor.rows
                For jcolumn = 1 To minor.columns
                    minor.element(jrow, jcolumn) = minorelements(arrayi)
                    arrayi += 1
                Next
            Next

            Return minor.determinant

        End Function

        Public Shared Operator +(matrix1 As CMatrix, matrix2 As CMatrix) As CMatrix
            If ((matrix1.rows <> matrix2.rows) Or (matrix1.columns <> matrix2.columns)) Then
                Throw New Exception("Matrices cannot be added unless they are the same size")
            Else
                Dim inter As New CMatrix(matrix1.rows, matrix1.columns)
                For row As Integer = 1 To matrix1.rows
                    For column As Integer = 1 To matrix1.columns
                        inter.element(row, column) = matrix1.element(row, column) + matrix2.element(row, column)
                    Next
                Next
                Return inter
            End If
        End Operator

        Public Shared Operator -(matrix1 As CMatrix, matrix2 As CMatrix) As CMatrix
            If ((matrix1.rows <> matrix2.rows) Or (matrix1.columns <> matrix2.columns)) Then
                Throw New Exception("Matrices cannot be subtracted unless they are the same size")
            Else
                Dim inter As New CMatrix(matrix1.rows, matrix1.columns)
                For row As Integer = 1 To matrix1.rows
                    For column As Integer = 1 To matrix1.columns
                        inter.element(row, column) = matrix1.element(row, column) - matrix2.element(row, column)
                    Next
                Next
                Return inter
            End If
        End Operator

        Public Shared Operator *(scalar As Complex, matrix1 As CMatrix) As CMatrix
            Dim inter As New CMatrix(matrix1.rows, matrix1.columns)
            For row As Integer = 1 To matrix1.rows
                For column As Integer = 1 To matrix1.columns
                    inter.element(row, column) = scalar * matrix1.element(row, column)
                Next
            Next
            Return inter
        End Operator

        Public Shared Operator *(matrix1 As CMatrix, matrix2 As CMatrix) As CMatrix
            If (matrix1.columns <> matrix2.rows) Then
                Throw New Exception("Matrices cannot be multiplied unless the number of columns in the left matrix equals the number of rows in the right matrix.")
            Else
                Dim inter As New CMatrix(matrix1.rows, matrix2.columns)
                For resultrow As Integer = 1 To inter.rows
                    For resultcolumn As Integer = 1 To inter.columns
                        For position As Integer = 1 To matrix1.columns
                            inter.element(resultrow, resultcolumn) += (matrix1.element(resultrow, position) * matrix2.element(position, resultcolumn))
                        Next
                    Next
                Next
                Return inter
            End If
        End Operator

    End Class

End Namespace
