﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class Circle

        Private _radius As Double

        Public Sub New()
            _radius = 0.0R
        End Sub

        Public Sub New(radius As Double)
            _radius = radius
        End Sub

        Public Property Radius As Double
            Get
                Return _radius
            End Get
            Set(value As Double)
                _radius = value
            End Set
        End Property

        Public Property Diameter As Double
            Get
                Return 2.0R * _radius
            End Get
            Set(value As Double)
                _radius = value / 2.0R
            End Set
        End Property

        Public Property Circumference As Double
            Get
                Return 2.0R * Math.PI * _radius
            End Get
            Set(value As Double)
                _radius = value / (2.0R * Math.PI)
            End Set
        End Property

        Public Property Area As Double
            Get
                Return Math.PI * (_radius ^ 2.0R)
            End Get
            Set(value As Double)
                _radius = Math.Sqrt(value / Math.PI)
            End Set
        End Property

        Public Function SubtendedArea(subtendedAngle As Angle) As Double
            Return (Me.Radius ^ 2.0R) * subtendedAngle.Radians / 2.0R
        End Function

        Public Function SubtendedArea(arclength As Double) As Double
            Return Me.Area * arclength / Me.Circumference
        End Function

        Public Function SubtendingArcLength(subtendedAngle As Angle) As Double
            Return Me.Radius * subtendedAngle.Radians
        End Function

        Public Function AreaAboveChord(distanceFromCenter As Double) As Double
            Dim theta As Angle = New Angle(2.0R * Math.Asin(distanceFromCenter / Me.Radius))
            Return ((Me.Radius ^ 2.0R) / 2.0R) * (theta.Radians - GMath.Sin(theta))
        End Function

        Public Shared Function AreaAboveChord(chordLength As Double, distanceFromArc As Double) As Double
            Dim radius As Double = (distanceFromArc / 2.0R) + ((chordLength ^ 2) / (8.0R * distanceFromArc))
            Return New Circle(radius).AreaAboveChord(radius - distanceFromArc)
        End Function

        Public Function CircleFromChord(chordLength As Double, distanceFromArc As Double) As Circle
            Return New Circle((distanceFromArc / 2.0R) + ((chordLength ^ 2) / (8.0R * distanceFromArc)))
        End Function

    End Class

End Namespace
