﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme, and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class Complex

        Private _re As Double
        Private _im As Double

        Public Sub New()

            _re = 0.0R
            _im = 0.0R

        End Sub

        Public Sub New(realpart As Double)

            _re = realpart
            _im = 0.0R

        End Sub

        Public Sub New(realpart As Double, imaginarypart As Double)

            _re = realpart
            _im = imaginarypart

        End Sub

        Public Sub New(magnitude As Double, phase As Angle)

            _re = magnitude * Math.Cos(phase.Radians)
            _im = magnitude * Math.Sin(phase.Radians)

        End Sub

        Public Property Re As Double
            Get
                Return _re
            End Get
            Set(value As Double)
                _re = value
            End Set
        End Property

        Public Property Im As Double
            Get
                Return _im
            End Get
            Set(value As Double)
                _im = value
            End Set
        End Property

        Public Property Magnitude As Double
            Get
                Return Math.Sqrt((_re ^ 2.0R) + (_im ^ 2.0R))
            End Get
            Set(value As Double)
                Dim workangle As Angle = New Angle(Me.Phase.Radians)
                Dim workcomplex As New Complex(value, workangle)
                _re = workcomplex.Re
                _im = workcomplex.Im
            End Set
        End Property

        Public Property Phase As Angle
            Get
                Return New Angle(Math.Atan2(_im, _re))
            End Get
            Set(value As Angle)
                Dim workmagnitude As Double = Me.Magnitude
                Dim workcomplex As New Complex(workmagnitude, value)
                _re = workcomplex.Re
                _im = workcomplex.Im
            End Set
        End Property

        Public ReadOnly Property Conjugate As Complex
            Get
                Return New Complex(_re, -_im)
            End Get
        End Property

        Public ReadOnly Property Reciprocal As Complex
            Get
                Return New Complex(1) / Me
            End Get
        End Property

        Public Shared ReadOnly Property I As Complex
            Get
                Return New Complex(0, 1)
            End Get
        End Property

        Public Shared ReadOnly Property zero As Complex 'new in 0.7.0
            Get
                Return New Complex(0, 0)
            End Get
        End Property

        Public Overrides Function ToString() As String
            Return Me.Re.ToString & " " & If(Me.Im >= 0.0R, "+", "-") & " " & Math.Abs(Me.Im).ToString & "i"
        End Function

        Public Shared Operator +(z1 As Complex, z2 As Complex) As Complex
            Return New Complex(z1.Re + z2.Re, z1.Im + z2.Im)
        End Operator

        Public Shared Operator +(z As Complex, r As Double) As Complex
            Return z + New Complex(r)
        End Operator

        Public Shared Operator +(r As Double, z As Complex) As Complex
            Return New Complex(r) + z
        End Operator

        Public Shared Operator -(z1 As Complex, z2 As Complex) As Complex
            Return New Complex(z1.Re - z2.Re, z1.Im - z2.Im)
        End Operator

        Public Shared Operator -(z As Complex, r As Double) As Complex
            Return z - New Complex(r)
        End Operator

        Public Shared Operator -(r As Double, z As Complex) As Complex
            Return New Complex(r) - z
        End Operator

        Public Shared Operator -(z As Complex) As Complex
            Return New Complex(-z.Re, -z.Im)
        End Operator

        Public Shared Operator *(z1 As Complex, z2 As Complex) As Complex
            Return New Complex((z1.Re * z2.Re) - (z1.Im * z2.Im), (z1.Re * z2.Im) + (z1.Im * z2.Re))
        End Operator

        Public Shared Operator *(z As Complex, r As Double) As Complex
            Return z * New Complex(r)
        End Operator

        Public Shared Operator *(r As Double, z As Complex) As Complex
            Return New Complex(r) * z
        End Operator

        Public Shared Operator /(z1 As Complex, z2 As Complex) As Complex
            Dim a As Double = z1.Re
            Dim b As Double = z1.Im
            Dim c As Double = z2.Re
            Dim d As Double = z2.Im
            Dim denom As Double = (c ^ 2.0R) + (d ^ 2.0R)

            Return New Complex(((a * c) + (b * d)) / denom, ((b * c) - (a * d)) / denom)

        End Operator

        Public Shared Operator /(z As Complex, r As Double) As Complex
            Return z / New Complex(r)
        End Operator

        Public Shared Operator /(r As Double, z As Complex) As Complex
            Return New Complex(r) / z
        End Operator

        Public Shared Operator ^(z1 As Complex, z2 As Complex) As Complex

            Dim z As Complex = z2 * GMath.Log(z1)
            Return New Complex(Math.Exp(z.Re) * Math.Cos(z.Im), Math.Exp(z.Re) * Math.Sin(z.Im))

        End Operator

        Public Shared Operator ^(z As Complex, r As Double) As Complex
            Return z ^ New Complex(r)
        End Operator

        Public Shared Operator ^(r As Double, z As Complex) As Complex
            Return New Complex(r) ^ z
        End Operator

        Public Shared Operator =(z1 As Complex, z2 As Complex) As Boolean
            If ((z1.Re = z2.Re) And (z1.Im = z2.Im)) Then
                Return True
            Else
                Return False
            End If
        End Operator

        Public Shared Operator <>(z1 As Complex, z2 As Complex) As Boolean
            If ((z1.Re <> z2.Re) Or (z1.Im <> z2.Im)) Then
                Return True
            Else
                Return False
            End If
        End Operator

        Public Shared Widening Operator CType(ByVal d As Double) As Complex 'new in 0.7.0
            Return New Complex(d)
        End Operator

        Public Shared Widening Operator CType(ByVal i As Integer) As Complex 'new in 0.7.0
            Return New Complex(CDbl(i))
        End Operator

    End Class

End Namespace
