﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class GMath

        Structure SlopeIntercept
            Dim slope As Double
            Dim intercept As Double
        End Structure

        Public Shared Function Abs(z As Complex) As Double
            Return z.Magnitude
        End Function

        Public Shared Function Exp(z As Complex) As Complex
            Return New Complex(Math.E) ^ z
        End Function

        Public Shared Function LineEquation2D(point1 As Point, point2 As Point) As SlopeIntercept
            Dim m As Double
            Dim b As Double

            m = (point2.y - point1.y) / (point2.x - point1.x)
            b = point1.y - (m * point1.x)

            Dim eq As New SlopeIntercept
            eq.slope = m
            eq.intercept = b
            Return eq

        End Function

        Public Shared Function Log(z As Complex) As Complex
            Return New Complex(Math.Log(z.Magnitude), z.Phase.Radians)
        End Function

        Public Shared Function Log(z As Complex, base As Double) As Complex
            Return New Complex(Math.Log(z.Magnitude) / Math.Log(base), z.Phase.Radians / Math.Log(base))
        End Function

        Public Shared Function Log(z As Complex, base As Complex) As Complex
            Return (New Complex(z.Magnitude) / GMath.Log(base)) + (New Complex(z.Phase.Radians) / GMath.Log(base))
        End Function

        Public Shared Function Log10(z As Complex) As Complex
            Return New Complex(Math.Log10(z.Magnitude), z.Phase.Radians / Math.Log(10))
        End Function

        Public Shared Function Sqrt(z As Complex) As Complex
            Return New Complex(Math.Sqrt((z.Re + z.Magnitude) / 2.0R), Math.Sign(z.Im) * Math.Sqrt((z.Magnitude - z.Re) / 2.0R))
        End Function

        Public Shared Function Sin(z As Complex) As Complex
            Return New Complex(Math.Sin(z.Re) * Math.Cosh(z.Im), Math.Cos(z.Re) * Math.Sinh(z.Im))
        End Function

        Public Shared Function Sin(a As Angle) As Double
            Return Math.Sin(a.Radians)
        End Function

        Public Shared Function Cos(z As Complex) As Complex
            Return New Complex(Math.Cos(z.Re) * Math.Cosh(z.Im), Math.Sin(z.Re) * Math.Sinh(z.Im))
        End Function

        Public Shared Function Cos(a As Angle) As Double
            Return Math.Cos(a.Radians)
        End Function

        Public Shared Function Tan(z As Complex) As Complex
            Return GMath.Sin(z) / GMath.Cos(z)
        End Function

        Public Shared Function Tan(a As Angle) As Double
            Return Math.Tan(a.Radians)
        End Function

        Public Shared Function Sinh(z As Complex) As Complex
            Return New Complex(Math.Sinh(z.Re) * Math.Cos(z.Im), Math.Cosh(z.Re) * Math.Sin(z.Im))
        End Function

        Public Shared Function Cosh(z As Complex) As Complex
            Return New Complex(Math.Cosh(z.Re) * Math.Cos(z.Im), Math.Sinh(z.Re) * Math.Sin(z.Im))
        End Function

        Public Shared Function Tanh(z As Complex) As Complex
            Return GMath.Sinh(z) / GMath.Cosh(z)
        End Function

        Public Shared Function Asin(z As Complex) As Complex
            Return -Complex.I * GMath.Log((Complex.I * z) + GMath.Sqrt(1.0R - (z ^ 2.0R)))
        End Function

        Public Shared Function Asin(sine As Double) As Angle
            Return New Angle(Math.Asin(sine))
        End Function

        Public Shared Function Acos(z As Complex) As Complex
            Return -Complex.I * GMath.Log(z + GMath.Sqrt((z ^ 2.0R) - 1.0R))
        End Function

        Public Shared Function Acos(cosine As Double) As Angle
            Return New Angle(Math.Acos(cosine))
        End Function

        Public Shared Function ATan(z As Complex) As Complex
            Return Complex.I * (GMath.Log(1.0R - (Complex.I * z)) - GMath.Log(1.0R + (Complex.I * z))) / 2.0R
        End Function

        Public Shared Function ATan(tangent As Double) As Angle
            Return New Angle(Math.Atan(tangent))
        End Function

        Public Shared Function Asinh(z As Complex) As Complex
            Return GMath.Log(z + GMath.Sqrt((z ^ 2) + 1.0R))
        End Function

        Public Shared Function Acosh(z As Complex) As Complex
            Return GMath.Log(z + (GMath.Sqrt(z + 1.0R) * GMath.Sqrt(z - 1.0R)))
        End Function

        Public Shared Function Atanh(z As Complex) As Complex
            Return (GMath.Log(1.0R + z) - GMath.Log(1.0R - z)) / 2.0R
        End Function

    End Class

End Namespace
