﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class Point

        Private _x As Double
        Private _y As Double
        Private _z As Double

        Public Enum heightangle
            elevation = 0
            inclination = 1
        End Enum

        ''' <summary>
        ''' This constructor generates a point at the origin
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub New()
            _x = 0.0R
            _y = 0.0R
            _z = 0.0R
        End Sub


        ''' <summary>
        ''' Use this constructor for points in rectilinear (Cartesian) coordinates
        ''' </summary>
        ''' <param name="x_coordinate"></param>
        ''' <param name="y_coordinate"></param>
        ''' <param name="z_coordinate"></param>
        ''' <remarks></remarks>
        Public Sub New(x_coordinate As Double, y_coordinate As Double, z_coordinate As Double)

            _x = x_coordinate
            _y = y_coordinate
            _z = z_coordinate

        End Sub

        ''' <summary>
        ''' Use this constructor for points in rectilinear (Cartesian) coordinates in the xy-plane
        ''' </summary>
        ''' <param name="x_coordinate"></param>
        ''' <param name="y_coordinate"></param>
        ''' <remarks></remarks>
        Public Sub New(x_coordinate As Double, y_coordinate As Double)

            _x = x_coordinate
            _y = y_coordinate
            _z = 0.0R

        End Sub

        ''' <summary>
        ''' Use this constructor for points in spherical coordinates
        ''' </summary>
        Public Sub New(r As Double, azimuth As Angle, polarAngle As Angle, Optional useInclination As Boolean = True) 'spherical coordinates

            Dim inclination As Angle

            If useInclination Then
                inclination = polarAngle
            Else
                inclination = New Angle((Math.PI / 2.0R) - polarAngle.Radians)
            End If

            _x = r * Math.Cos(azimuth.Radians) * Math.Sin(inclination.Radians)
            _y = r * Math.Sin(azimuth.Radians) * Math.Sin(inclination.Radians)
            _z = r * Math.Cos(azimuth.Radians)


        End Sub

        ''' <summary>
        ''' Use this constructor for points in spherical coordinates
        ''' </summary>
        Public Sub New(r As Double, azimuth As Angle, polarAngle As Angle, heightAngleType As heightangle) 'spherical coordinates

            Dim inclination As Angle

            If heightAngleType = heightangle.inclination Then
                inclination = polarAngle
            Else
                inclination = New Angle((Math.PI / 2.0R) - polarAngle.Radians)
            End If

            _x = r * Math.Cos(azimuth.Radians) * Math.Sin(inclination.Radians)
            _y = r * Math.Sin(azimuth.Radians) * Math.Sin(inclination.Radians)
            _z = r * Math.Cos(azimuth.Radians)


        End Sub

        ''' <summary>
        ''' Use this constructor for points in cylindrical coordinates
        ''' </summary>
        Public Sub New(rho As Double, azimuth As Angle, z As Double) 'cylindrical coordinates

            _x = rho * Math.Cos(azimuth.Radians)
            _y = rho * Math.Sin(azimuth.Radians)
            _z = z

        End Sub

        Public Property x As Double
            Get
                Return _x
            End Get
            Set(value As Double)
                _x = x
            End Set
        End Property

        Public Property y As Double
            Get
                Return _y
            End Get
            Set(value As Double)
                _y = y
            End Set
        End Property

        Public Property z As Double
            Get
                Return _z
            End Get
            Set(value As Double)
                _z = z
            End Set
        End Property

        ''' <summary>
        ''' This is the spherical radius
        ''' </summary>
        Public Property r As Double
            Get
                Return Math.Sqrt((_x ^ 2) + (_y ^ 2) + (_z ^ 2))
            End Get
            Set(value As Double)
                Dim workpoint As New Point(value, Me.azimuth, Me.inclination, True)
                _x = workpoint.x
                _y = workpoint.y
                _z = workpoint.z
            End Set
        End Property

        Public Property azimuth As Angle
            Get
                Return New Angle(Math.Atan2(_y, _x))
            End Get
            Set(value As Angle)
                Dim workpoint As New Point(Me.rho, value, Me.z)
                _x = workpoint.x
                _y = workpoint.y
                _z = workpoint.z
            End Set
        End Property

        Public Property inclination As Angle
            Get
                Return New Angle(Math.Acos(_z / (Math.Sqrt((_x ^ 2) + (_y ^ 2) + (_z ^ 2)))))
            End Get
            Set(value As Angle)
                Dim workpoint As New Point(Me.r, Me.azimuth, value, True)
                _x = workpoint.x
                _y = workpoint.y
                _z = workpoint.z
            End Set
        End Property

        Public Property elevation As Angle
            Get
                Return New Angle((Math.PI / 2) - Me.inclination.Radians)
            End Get
            Set(value As Angle)
                Dim workpoint As New Point(Me.r, Me.azimuth, value, False)
                _x = workpoint.x
                _y = workpoint.y
                _z = workpoint.z
            End Set
        End Property

        ''' <summary>
        ''' This is the cylindrical radius
        ''' </summary>
        Public Property rho As Double
            Get
                Return Math.Sqrt((_x ^ 2) + (_y ^ 2))
            End Get
            Set(value As Double)
                Dim workpoint As New Point(value, Me.azimuth, Me.z)
                _x = workpoint.x
                _y = workpoint.y
                _z = workpoint.z
            End Set
        End Property

        Public ReadOnly Property DistanceFrom(p As Point) As Double
            Get
                Return Math.Sqrt(((p.x - Me.x) ^ 2.0R) + ((p.y - Me.y) ^ 2.0R) + ((p.z - Me.z) ^ 2.0R))
            End Get
        End Property

        Public ReadOnly Property DistanceFromOrigin As Double
            Get
                Return Math.Sqrt((Me.x ^ 2.0R) + (Me.y ^ 2.0R) + (Me.z ^ 2.0R))
            End Get
        End Property

        Public ReadOnly Property ToVector As Vector
            Get
                Return New Vector(Me.x, Me.y, Me.z)
            End Get
        End Property

        Public ReadOnly Property RectCoord As String
            Get
                Return "(" & Me.x.ToString & ", " & Me.y.ToString & ", " & Me.z.ToString & ")"
            End Get
        End Property

        Public ReadOnly Property SphCoord(angleUnit As Angle.angleunit, heightAngleType As Point.heightangle)

            Get
                Dim azimuth As Double
                Dim polarAngle As Double
                Dim angleunittext As String

                If heightAngleType = heightangle.inclination Then
                    If angleUnit = Angle.angleunit.radians Then
                        polarAngle = Me.inclination.Radians
                    ElseIf angleUnit = Angle.angleunit.degrees Then
                        polarAngle = Me.inclination.Degrees
                    ElseIf angleUnit = Angle.angleunit.grads Then
                        polarAngle = Me.inclination.Grads
                    ElseIf angleUnit = Angle.angleunit.revolutions Then
                        polarAngle = Me.inclination.Revolutions
                    End If
                Else
                    If angleUnit = Angle.angleunit.radians Then
                        polarAngle = Me.elevation.Radians
                    ElseIf angleUnit = Angle.angleunit.degrees Then
                        polarAngle = Me.elevation.Degrees
                    ElseIf angleUnit = Angle.angleunit.grads Then
                        polarAngle = Me.elevation.Grads
                    ElseIf angleUnit = Angle.angleunit.revolutions Then
                        polarAngle = Me.elevation.Revolutions
                    End If
                End If

                If angleUnit = Angle.angleunit.radians Then
                    azimuth = Me.azimuth.Radians
                    angleunittext = "rad"
                ElseIf angleUnit = Angle.angleunit.degrees Then
                    azimuth = Me.azimuth.Degrees
                    angleunittext = "deg"
                ElseIf angleUnit = Angle.angleunit.grads Then
                    azimuth = Me.azimuth.Grads
                    angleunittext = "gra"
                ElseIf angleUnit = Angle.angleunit.revolutions Then
                    azimuth = Me.azimuth.Revolutions
                    angleunittext = "rev"
                End If

                If heightAngleType = heightangle.inclination Then
                    Return "(rad = " & Me.r.ToString & ", azi = " & azimuth.ToString & " " & angleunittext & ", inc = " & polarAngle.ToString & " " & angleunittext & ")"
                Else
                    Return "(rad = " & Me.r.ToString & ", azi = " & azimuth.ToString & " " & angleunittext & ", ele = " & polarAngle.ToString & " " & angleunittext & ")"
                End If

            End Get

        End Property

        Public ReadOnly Property SphCoord(Optional angleUnit As String = "rad", Optional inclination As Boolean = True) As String
            Get
                Dim azimuth As Double
                Dim polarAngle As Double

                If inclination Then
                    If angleUnit = "rad" Then
                        polarAngle = Me.inclination.Radians
                    ElseIf angleUnit = "deg" Then
                        polarAngle = Me.inclination.Degrees
                    ElseIf angleUnit = "gra" Then
                        polarAngle = Me.inclination.Grads
                    ElseIf angleUnit = "rev" Then
                        polarAngle = Me.inclination.Revolutions
                    End If
                Else
                    If angleUnit = "rad" Then
                        polarAngle = Me.elevation.Radians
                    ElseIf angleUnit = "deg" Then
                        polarAngle = Me.elevation.Degrees
                    ElseIf angleUnit = "gra" Then
                        polarAngle = Me.elevation.Grads
                    ElseIf angleUnit = "rev" Then
                        polarAngle = Me.elevation.Revolutions
                    End If
                End If

                If angleUnit = "rad" Then
                    azimuth = Me.azimuth.Radians
                ElseIf angleUnit = "deg" Then
                    azimuth = Me.azimuth.Degrees
                ElseIf angleUnit = "gra" Then
                    azimuth = Me.azimuth.Grads
                ElseIf angleUnit = "rev" Then
                    azimuth = Me.azimuth.Revolutions
                End If

                If inclination Then
                    Return "(rad = " & Me.r.ToString & ", azi = " & azimuth.ToString & " " & angleUnit & ", inc = " & polarAngle.ToString & " " & angleUnit & ")"
                Else
                    Return "(rad = " & Me.r.ToString & ", azi = " & azimuth.ToString & " " & angleUnit & ", ele = " & polarAngle.ToString & " " & angleUnit & ")"
                End If

            End Get
        End Property

        Public ReadOnly Property CylCoord(Optional angleUnit As String = "rad") As String
            Get
                Dim azimuth As Double

                If angleUnit = "rad" Then
                    azimuth = Me.azimuth.Radians
                ElseIf angleUnit = "deg" Then
                    azimuth = Me.azimuth.Degrees
                ElseIf angleUnit = "gra" Then
                    azimuth = Me.azimuth.Grads
                ElseIf angleUnit = "rev" Then
                    azimuth = Me.azimuth.Revolutions
                End If

                Return "(rho = " & Me.rho.ToString & ", azi = " & azimuth.ToString & " " & angleUnit & ", z = " & z.ToString & ")"

            End Get
        End Property

        Public ReadOnly Property CylCoord(angleUnit As Angle.angleunit) As String
            Get
                Dim azimuth As Double
                Dim angleunittext As String

                If angleUnit = Angle.angleunit.radians Then
                    azimuth = Me.azimuth.Radians
                    angleunittext = "rad"
                ElseIf angleUnit = Angle.angleunit.degrees Then
                    azimuth = Me.azimuth.Degrees
                    angleunittext = "deg"
                ElseIf angleUnit = Angle.angleunit.grads Then
                    azimuth = Me.azimuth.Grads
                    angleunittext = "gra"
                ElseIf angleUnit = Angle.angleunit.revolutions Then
                    azimuth = Me.azimuth.Revolutions
                    angleunittext = "rev"
                End If

                Return "(rho = " & Me.rho.ToString & ", azi = " & azimuth.ToString & " " & angleunittext & ", z = " & z.ToString & ")"

            End Get
        End Property

        Public Function Translate(displacement As Vector) As Point
            Return (Me.ToVector + displacement).ToPoint
        End Function

        Public Function Translate(currentOrigin As Point, newOrigin As Point) As Point
            Return (Me.ToVector + Point.MakeVector(currentOrigin, newOrigin)).ToPoint
        End Function

        Public Function Translate(newOrigin As Point) As Point
            Return (Me.ToVector + newOrigin.ToVector).ToPoint
        End Function

        Public Shared Function ApexAngle(legPoint1 As Point, apexPoint As Point, legPoint2 As Point) As Angle
            Return Vector.AngleBetweenVectors(MakeVector(apexPoint, legPoint1), MakeVector(apexPoint, legPoint2))
        End Function

        Public Shared Function MakeVector(startpoint As Point, endpoint As Point) As Vector
            Return New Vector(endpoint.x - startpoint.x, endpoint.y - startpoint.y, endpoint.z - startpoint.z)
        End Function

    End Class

End Namespace
