﻿'GEOMETER:  A .NET class library written in Visual Basic Express 2012
'for analytic geometry, particularly vectors and complex numbers.

'Version 0.7.0 April 19 2013
'Copyright 2013 Wayne Matthew Syvinski

'TERMS OF USE

'*Disclaimer of Warranty

'    This software library (Geometer) comes with absolutely, positively NO WARRANTY, neither express or implied,  
'not even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. You use this software at your own risk.  
'In case of loss, no one owes you anything whatsoever. You have been warned. There is also NO GUARANTEE of 
'MATHEMATICAL CORRECTNESS. If you are going to use this software library in a production environment, 
'it is YOUR RESPONSIBILITY to verify that the methods, properties, and operators  return the correct values. 

'*Licensing

'    Geometer has a dual licensing scheme , and a patent provision.

'    If you use Geometer as-is, or make modifications to it other than making any class inheritable, 
'you may use it under the terms of the GNU Lesser General Public License version 3.

'    However, if you amend any class definition so as to make the class inheritable, you must use it under 
'the terms of the GNU General Public License version 3.

'    The GNU Lesser General Public License and the GNU General Public License are available at http://www.fsf.org

'    In addition, this software may not be incorporated into any software, whether as source or as linked library, 
'that is encumbered under any patent, unless all such patents have been released into the public domain.

'    When using Geometer under either aforementioned license, credit must be given to the copyright holder.

'    The copyright holder reserves the right to license Geometer under subsequent versions of the 
'GNU Lesser General Public License and the GNU General Public License.

'*Naming of Derivative Works

'    Both the GNU Lesser General Public License and the GNU General Public License allow developers to create derivative works.  
'However, if you create a derivative work, you must use a namespace other than geometer.  
'In addition, your derivative work may not be called "Geometer" or use the word geometer in its title.

Namespace geometer

    Public NotInheritable Class Triangle

        Private _pointA As Point
        Private _pointB As Point
        Private _pointC As Point

        Public Sub New()
            _pointA = New Point()
            _pointB = New Point()
            _pointC = New Point()
        End Sub

        Public Sub New(pointA As Point, pointB As Point, pointC As Point)

            _pointA = pointA
            _pointB = pointB
            _pointC = pointC

        End Sub
        ''' <summary>
        ''' A Triangle object made using this constructor will represent the triangle in the xy-plane with point A at the origin and side C on the positive x-axis
        ''' </summary>
        Public Sub New(sideA As Double, sideB As Double, sideC As Double)

            Dim azimuthAngle As Double = Math.Acos(((sideB ^ 2) + (sideC ^ 2) - (sideA ^ 2)) / (2 * sideB * sideC))

            _pointA = New Point(0.0R, 0.0R, 0.0R)
            _pointB = New Point(sideC, 0.0R, 0.0R)
            _pointC = New Point(sideB, New Angle(azimuthAngle), 0.0R)

        End Sub

        Public ReadOnly Property Area As Double
            Get
                Return Point.MakeVector(_pointA, _pointB).Cross(Point.MakeVector(_pointA, _pointC)).Magnitude / 2.0R
            End Get
        End Property

        Public ReadOnly Property Perimeter As Double
            Get
                Return _pointA.DistanceFrom(_pointB) + _pointB.DistanceFrom(_pointC) + _pointC.DistanceFrom(_pointA)
            End Get
        End Property

        Public ReadOnly Property alpha As Angle
            Get
                Return Vector.AngleBetweenVectors(Point.MakeVector(_pointA, _pointB), Point.MakeVector(_pointA, _pointC))
            End Get
        End Property

        Public ReadOnly Property beta As Angle
            Get
                Return Vector.AngleBetweenVectors(Point.MakeVector(_pointB, _pointC), Point.MakeVector(_pointB, _pointA))
            End Get
        End Property

        Public ReadOnly Property gamma As Angle
            Get
                Return Vector.AngleBetweenVectors(Point.MakeVector(_pointC, _pointA), Point.MakeVector(_pointC, _pointB))
            End Get
        End Property

        Public Property ptA As Point
            Get
                Return _pointA
            End Get
            Set(value As Point)
                _pointA = value
            End Set
        End Property

        Public Property ptB As Point
            Get
                Return _pointB
            End Get
            Set(value As Point)
                _pointB = value
            End Set
        End Property

        Public Property ptC As Point
            Get
                Return _pointC
            End Get
            Set(value As Point)
                _pointC = value
            End Set
        End Property

        Public ReadOnly Property sideA As Double
            Get
                Return Me.ptB.DistanceFrom(Me.ptC)
            End Get
        End Property

        Public ReadOnly Property sideB As Double
            Get
                Return Me.ptC.DistanceFrom(ptA)
            End Get
        End Property

        Public ReadOnly Property sideC As Double
            Get
                Return Me.ptA.DistanceFrom(ptB)
            End Get
        End Property

        Public ReadOnly Property UnitNormal As Vector
            Get
                Dim workvector As Vector = Point.MakeVector(Me.ptA, Me.ptB).Cross(Point.MakeVector(Me.ptA, Me.ptC))

                If workvector.k < 0.0R Then
                    workvector = -workvector
                ElseIf workvector.k = 0.0R Then
                    If workvector.j < 0.0R Then
                        workvector = workvector
                    ElseIf workvector.j = 0.0R Then
                        If workvector.i < 0.0R Then
                            workvector = -workvector
                        End If
                    End If
                End If
                Return workvector.UnitVector
            End Get
        End Property

        Public ReadOnly Property SineRuleRatio As Double
            Get
                Dim ratioA As Double = Me.sideA / GMath.Sin(Me.alpha)
                Dim ratioB As Double = Me.sideB / GMath.Sin(Me.beta)
                Dim ratioC As Double = Me.sideC / GMath.Sin(Me.gamma)
                Return (ratioA + ratioB + ratioC) / 3.0R
            End Get
        End Property

        Public ReadOnly Property AltitudeA As Double
            Get
                Dim s As Double = Me.Perimeter / 2.0R
                Return 2.0R * Math.Sqrt(s * (s - Me.sideA) * (s - Me.sideB) * (s - Me.sideC)) / sideA
            End Get
        End Property

        Public ReadOnly Property AltitudeB As Double
            Get
                Dim s As Double = Me.Perimeter / 2.0R
                Return 2.0R * Math.Sqrt(s * (s - Me.sideA) * (s - Me.sideB) * (s - Me.sideC)) / sideB
            End Get
        End Property

        Public ReadOnly Property AltitudeC As Double
            Get
                Dim s As Double = Me.Perimeter / 2.0R
                Return 2.0R * Math.Sqrt(s * (s - Me.sideA) * (s - Me.sideB) * (s - Me.sideC)) / sideC
            End Get
        End Property

        Public ReadOnly Property Centroid As Point
            Get
                Return ((Me.ptA.ToVector + Me.ptB.ToVector + Me.ptC.ToVector) / 3.0R).ToPoint
            End Get
        End Property

        Public ReadOnly Property midptA As Point
            Get
                Return ((Me.ptB.ToVector + Me.ptC.ToVector) / 2.0R).ToPoint
            End Get
        End Property

        Public ReadOnly Property midptB As Point
            Get
                Return ((Me.ptA.ToVector + Me.ptC.ToVector) / 2.0R).ToPoint
            End Get
        End Property

        Public ReadOnly Property midptC As Point
            Get
                Return ((Me.ptB.ToVector + Me.ptA.ToVector) / 2.0R).ToPoint
            End Get
        End Property


    End Class

End Namespace

